<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version $Id: Validator.php 4537 2010-08-24 13:11:27Z tperdue $
 * @author Marcelo Mottalli	<mottalli@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

/**
 * A collection of system utilities for validation and sanitizing of strings
 */

require_once('StringUtils.php');
require_once('SystemUtils.php');
require_once('lib/external/HTMLSafe.php');

class Validator {
	public static function sanitizeInt($value) {
		return intval($value);
	}

	public static function sanitizeString($value) {
		return StringUtils::toValidUTF8(htmlspecialchars($value,ENT_COMPAT));
	}

	// $value can be other than True or False (like "true" or "false", 0 or 1)
	public static function sanitizeBool($value) {
		if (is_bool($value)) {
			return $value;
		}
		else if (is_string($value)) {
			$value = Validator::sanitizeString($value);
			if(strcasecmp($value, 'true')==0 || $value=='1' || strcasecmp($value, 't')==0 ) {
				return true;
			}
			else {
				return false;
			}
		}
		else {
			$value = Validator::sanitizeInt($value);
			if ($value == 0) {
				return false;
			}
			else {
				return true;
			}
		}
	}

	/**
	 * @todo	Finish this method
	 */
	public static function sanitizeDate($value) {
		$value = addslashes($value);
		if (trim($value)=="")
			return 0;
		if (!strtotime($value)) {
			// $value is an invalid date, or has invalid chars
			return 0;
		}
		return $value;
	}

	/**
	 * Validates the format of an email address
	 *
	 * This function is almost RFC 2822 compliant, the only
	 * case not allowed is escaped @ in the local name (ie: some\@user@domain.com
	 * However this syntax is deprecated and can be safely ignored
	 * @param string $email
	 * @return boolean
	 */
	public static function validateEmail($email) {
  		// First, we check that there's one @ symbol, 
  		// and that the lengths are right.
		if (!ereg("^[^@]{1,64}@[^@]{1,255}$", $email)) {
			// Email invalid because wrong number of characters 
			// in one section or wrong number of @ symbols.
			return false;
		}
		// Split it into sections to make life easier
		$email_array = explode("@", $email);
		$local_array = explode(".", $email_array[0]);
		for ($i = 0; $i < sizeof($local_array); $i++) {
			if(!ereg("^(([A-Za-z0-9!#$%&'*+/=?^_`{|}~-][A-Za-z0-9!#$%&'*+/=?^_`{|}~\.-]{0,63})|(\"[^(\\|\")]{0,62}\"))$",
			$local_array[$i])) {
				return false;
			}
		}
		// Check if domain is IP. If not, 
		// it should be valid domain name
		if (!ereg("^\[?[0-9\.]+\]?$", $email_array[1])) {
			$domain_array = explode(".", $email_array[1]);
			if (sizeof($domain_array) < 2) {
				return false; // Not enough parts to domain
			}
			for ($i = 0; $i < sizeof($domain_array); $i++) {
				if(!ereg("^(([A-Za-z0-9][A-Za-z0-9-]{0,61}[A-Za-z0-9])|([A-Za-z0-9]+))$",$domain_array[$i])) {
					return false;
				}
		 	}
		}
			return true;

	}

	/**
	 * Validates a list of emails.
	 * @param	string	$emailList
	 * @param	string	$separator	String used as a separator in the list.
	 * @return	array	Array with the invalid emails. If no invalid emails are found, it returns an empty array.
	 */
	public static function validateEmailList($list, $separator=',') {
		$emails = explode($separator, $list);
		$invalid = array();
		foreach ($emails as $email) {
			$email = trim($email);
			if (!self::validateEmail($email)) {
				$invalid[] = $email;
			}
		}

		return $invalid;
	}

	public static function validateSystemName($value) {
		return !preg_match('/[^[:alnum:]_]/',$value);
	}

	public static function validateFileName($value) {
		//@TODO add more allowed chars
		$invalidchars = eregi_replace("[-A-Z0-9_\.]","",$value);
		if(!empty($invalidchars) || $value == ".." || $value == "." || !$value) return false;
		return true;
	}

	public static function validateDate($value) {
		return true; //@TODO
	}

	/**
	 * Checks whether a password is valid. Valid passwords are 6+ alphanumeric characters
	 *
	 * @param string $password
	 * @return array	first value is true or false. second value is error msg array in case false
	 */
	public static function validatePassword($password) {
		$return = array();
		$return[0] = true;
		$return[1] = array();
		if (!preg_match('/'.GFConf::get('system.passwordregex').'/', $password)) {
			$return[0] = false;
			$return[1][] = Language::getSessText('Validation.PasswordInvalid');
		}
		return $return;
	}

	/**
	 * Checks that a POST action is valid (usually used in the Action scripts)
	 * @param	bool	True for checking if the user is logged in
	 */
	public static function validatePOSTAction($validate_logged_in = true) {
		if (!isset($_SERVER['REQUEST_METHOD']) || $_SERVER['REQUEST_METHOD'] != 'POST') {
			throw new PermissionDeniedException();
		}

		if ($validate_logged_in) {
			$sess = Session::getSession();
			if (!$sess->isLoggedIn()) {
				throw new NotLoggedInException();
			}
		}
	}

	/**
	 *  SanitizeHtml - Grabs some text with all kinds of html code and parses it to make it safe
	 *
	 *	@param   string		The HTML Code
	 *	@return  string		The HTML output
	 */
	public static function sanitizeHtml($input) {
	
		$sanitizer = new HTML_Safe();
		$input = $sanitizer->parse($input);
		
		
		return $input;
	}

	/**
	 * Validate a redirect URL
	 */
	public static function validateRedirect($url) {
		if ($url == "/") {
			return true;
		}elseif (!preg_match('/[:<>]/', $url) && preg_match('/^\/gf\//',$url)) {
			return true;
		}else{
			return false;
		}
	}


	/**
	 * Makes sure the URI used to invoke an action doesn't contain invalid characters
	 */
	public static function validateLocalUri($url) {
		return !preg_match('/[:<>]/', $url);
	}

	public static function validateURL($url) {
		return preg_match('/^https?:\\/\\//i', $url);
	}
}
?>
