<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('lib/exceptions/FilePermissionsException.php');

/**
 * This class allows reading and writing Perforce configuration files. The file must be stored
 * somewhere on the local filesystem
 */
class PerforceConfigFile {
	private $variables = array();
	private $filename;
	
	/**
	 * @parameter	string $filename	File where the configuration is stored. If it doesn't exist, it's created.
	 */
	public function __construct($filename) {
		$this->filename = $filename;
		
		$this->readConfiguration();
	}
	
	public function setVar($varname, $value) {
		$varname = strtolower($varname);
		$this->variables[$varname] = $value;
	}
	
	public function getVar($varname) {
		$varname = strtolower($varname);
		if (!array_key_exists($varname, $this->variables)) {
			return null;
		}
		
		return $this->variables[$varname];
	}
	
	/**
	 * Save the changes to the configuration file
	 */
	public function save() {
		$fh = fopen($this->filename, 'w');
		if (!$fh) {
			throw new FilePermissionException('Could not open '.$this->filename.' for writing');
		}
		
		$contents = array();
		
		foreach ($this->variables as $var => $value) {
			if (is_array($value)) {
				$contents[] = $var.':';
				foreach ($value as $item) {
					$contents[] = "\t".$item;
				}
			} else {
				$contents[] = $var.':'."\t".$value;
			}
			
		}
		
		fwrite($fh, implode("\n", $contents)."\n");
		fclose($fh);
	}
	
	private function readConfiguration() {
		$lines = file($this->filename);
		if (!is_array($lines)) {
			throw new FilePermissionException('Could not open '.$this->filename.' for reading');
		}
		
		$n = count($lines);
		$curr_var = '';
		for ($i=0; $i < $n; $i++) {
			$line = trim($lines[$i]);
			
			// comment
			if (strpos($line, '#') === 0) {
				continue;
			}
			
			if (preg_match('/([[:alnum:]]*):[[:blank:]]*(.+)$/', $line, $matches)) {
				// Found a "var: value" pair
				$this->variables[strtolower($matches[1])] = $matches[2];
			} else if (preg_match('/([[:alnum:]]*):$/', $line, $matches)) {
				// A line with just a variable name. Means that either the
				// value is empty, that the value is in the following line 
				// or that it has several values
				$curr_var = strtolower($matches[1]);		// For now we'll store the values in an array
				$this->variables[$curr_var] = array();
			} else if (!empty($line) && !empty($curr_var)) {
				// A value linked to the current variable
				$this->variables[$curr_var][] = $line;
			}
		}
		
		// Assume that one-liners arrays are just strings and empty arrays are empty strings
		$vars = array_keys($this->variables);
		foreach ($vars as $var) {
			if (is_array($this->variables[$var])) {
				if (count($this->variables[$var]) == 0) {
					$this->variables[$var] = '';
				} else if (count($this->variables[$var]) == 1) {
					$this->variables[$var] = $this->variables[$var][0];
				}
			}
		}
	}
}
?>